package shinh;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class YourHand extends Canvas {
	private final double PI = Math.PI;

	private final int KEY_UP = 1;
	private final int KEY_DOWN = 2;
	private final int KEY_RIGHT = 4;
	private final int KEY_LEFT = 8;

	private final int WIDTH = 200;
	private final int HEIGHT = 200;
	private final int WIDTH_2 = WIDTH/2;
	private final int HEIGHT_2 = HEIGHT/2;

	private String outputFile = "shinh/You.data/input";

	private int axis = 0;
	private int fire = 0;
	private int aim = 0;

	private void saveState() {
		try {
			int in = axis | fire;
			FileOutputStream output =
				new FileOutputStream(outputFile);
			output.write(in);
			output.write(aim);
			output.close();

			System.out.println(Integer.toString(axis) + " " + Integer.toString(aim));
		}
		catch (Exception ex) {
			System.out.println(ex.getStackTrace().toString());
		}
	}

	public void paint(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, WIDTH, HEIGHT);

		g.setColor(Color.BLACK);
		g.drawOval(WIDTH_2>>1, HEIGHT_2>>1, WIDTH_2, HEIGHT_2);
		g.drawLine(WIDTH_2, 0, WIDTH_2, HEIGHT);
		g.drawLine(0, HEIGHT_2, WIDTH, HEIGHT_2);
	}

	public YourHand() {
		setSize(WIDTH, HEIGHT);

		addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					fire = 16;
					saveState();
				}
				public void mouseReleased(MouseEvent e) {
					fire = 0;
					saveState();
				}
			}
			);

		addMouseMotionListener(new MouseMotionAdapter() {
				private final long inputInterval = 1000 / 30; // 30FPS
				private long prevInputTime = 0;

				public void mouseMoved(MouseEvent e) {
					long now = System.currentTimeMillis();
					if (now - prevInputTime < inputInterval) return;
					prevInputTime = now;

					double rad =
						Math.atan2(-(e.getX()-WIDTH_2), e.getY()-HEIGHT_2);
					aim = (int)((rad / PI + 1) * 128);
					if (aim > 255) aim = 0;
					else if (aim < 0) aim = 255;

					saveState();
				}
			}
			);

		addKeyListener(new KeyAdapter() {
				private YourHand you;

				public void keyReleased(KeyEvent e) {
					int key = e.getKeyCode();
					if (key == KeyEvent.VK_UP) {
						axis &= ~KEY_UP;
					}
					if (key == KeyEvent.VK_DOWN) {
						axis &= ~KEY_DOWN;
					}
					if (key == KeyEvent.VK_RIGHT) {
						axis &= ~KEY_RIGHT;
					}
					if (key == KeyEvent.VK_LEFT) {
						axis &= ~KEY_LEFT;
					}
					if (key == KeyEvent.VK_SPACE) {
						fire = 0;
					}

					saveState();
				}

				public void keyPressed(KeyEvent e) {
					int key = e.getKeyCode();

					if (key == KeyEvent.VK_UP) {
						axis |= KEY_UP;
					}
					if (key == KeyEvent.VK_DOWN) {
						axis |= KEY_DOWN;
					}
					if (key == KeyEvent.VK_RIGHT) {
						axis |= KEY_RIGHT;
					}
					if (key == KeyEvent.VK_LEFT) {
						axis |= KEY_LEFT;
					}
					if (key == KeyEvent.VK_SPACE) {
						fire |= 16;
					}

					saveState();
				}
			}
			);
	}

	public static void main(String args[]) {
		Frame frame = new Frame();
		frame.setTitle("your hand");
		frame.add(new YourHand());
		frame.setResizable(false);
		frame.pack();
		frame.show();
	}
}
